<?php
/*
*  @author metacreo <metacreo@newestvision.com>
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

class SimpleSecurity extends Module
{
    private $_executed = false;

    private $_checkAttemptsIntervalMinutes = 1;

    private $_att = [
        'authentication' => [
            'warn' => 3,
            'ban' => 5
        ],
        'registration' => [
            'warn' => 3,
            'ban' => 5
        ],
        'contact' => [
            'warn' => 3,
            'ban' => 5
        ],
        'comment' => [
            'warn' => 3,
            'ban' => 5
        ],
        'iqitreview' => [
            'warn' => 3,
            'ban' => 5
        ],
        'newsletter' => [
            'warn' => 3,
            'ban' => 5
        ]
    ];

    private $pwd_use_hash = 1;

    private $_remote_addr = null;
    private $_http_referer = null;
    private $_http_user_agent = null;
    private $_http_accept = null;
    private $_http_accept_language = null;
    private $_http_cookie = null;
    private $_http_host = null;
    private $_http_x_requested_with = null;
    private $_php_self = null;
    private $_script_name = null;

    private $_block_ip = 0; // 0 - not blocked, 1 - blocked, 2 - never block
    private $_block_email = 0; // 0 - not blocked, 1 - blocked, 2 - never block

    public $_action = null;
    private $_attempt = 1;
    private $_detected = [];

    private $_email = null;
    private $_password = null;
    private $_message = null;

    private $_redirect = null;

    protected $_errors = [];

    protected $tabs = [
        [
            'name' => 'SS Triggers',
            'class_name' => 'AdminSimpleSecurityTrigger',
            'visible' => true,
            'parent_class_name' => 'AdminParentCustomer',
        ],
        [
            'name' => 'SS Actions',
            'class_name' => 'AdminSimpleSecurityAction',
            'visible' => true,
            'parent_class_name' => 'AdminParentCustomer',
        ]
    ];

    public function __construct()
    {
        $this->name = 'simplesecurity';
        $this->tab = 'administration';
        $this->version = '1.0.6';
        $this->author = 'metacreo';
        $this->need_instance = 0;
        $this->bootstrap = true;
        $this->ps_versions_compliancy = ['min' => '1.7.8.3', 'max' => _PS_VERSION_];
        parent::__construct();
        $this->displayName = 'SimpleSecurity';
        $this->description = 'Check and ban IP or Email addresses on registration, authentication, newsletter, comment, review, bots, attacks.';
        $this->init();
    }

    private function init()
    {
        if ($this->active) {
            foreach ($this->_att as $name => $action) {
                foreach (array_keys($action) as $actionName) {
                    $config = strtoupper($this->name . '_' . $name . '_' . $actionName);
                    $this->_att[$name][$actionName] = (int) Configuration::get($config);
                }
                if ($this->_att[$name]['warn'] >= $this->_att[$name]['ban']) {
                    $this->warning = 'Invalid configuration.';
                }
            }
            $this->pwd_use_hash = (int) Configuration::get('SIMPLESECURITY_PASSWORDS_HASH');
        }
    }

    public function install()
    {
        Configuration::updateValue('SIMPLESECURITY_PASSWORDS_HASH', 1);
        foreach ($this->_att as $name => $action) {
            foreach ($action as $actionName => $actionValue) {
                $config = strtoupper($this->name . '_' . $name . '_' . $actionName);
                Configuration::updateValue($config, $actionValue);
            }
        }
        $this->installModuleOverrides();
        $this->installSQL();
        return parent::install() &&
            $this->registerHook('actionNewsletterRegistrationBefore') &&
            $this->registerHook('actionAuthenticationBefore') &&
            $this->registerHook('actionSubmitAccountBefore');
    }

    public function uninstall()
    {
        foreach ($this->configNames() as $config) {
            Configuration::deleteByName($config);
        }
        $this->uninstallModuleOverrides();
        $this->uninstallSQL();
        return parent::uninstall();
    }

    public function enable($force_all = false)
    {
        $this->installModuleOverrides();
        return parent::enable($force_all);
    }

    public function disable($force_all = false)
    {
        $this->uninstallModuleOverrides();
        return parent::disable($force_all);
    }

    private function installSQL()
    {
        $queries = include dirname(__FILE__) . '/sql/install.php';
        if (is_array($queries)) {
            return $this->performInstallQueries($queries);
        }
    }

    private function uninstallSQL()
    {
        $queries = include dirname(__FILE__) . '/sql/uninstall.php';
        if (is_array($queries)) {
            return $this->performInstallQueries($queries);
        }
    }

    private function installModuleOverrides()
    {
        $psOverridePath = _PS_OVERRIDE_DIR_ . 'modules/';
        $modOverridePath = _PS_MODULE_DIR_ . $this->name . '/override/modules/';
        $supportedModules = [
            'productcomments',
            'iqitreviews'
        ];
        foreach ($supportedModules as $module) {
            if (Module::isEnabled($module)) {
                $this->recursiveCopy($modOverridePath . $module, $psOverridePath . $module);
            }
        }
    }

    private function uninstallModuleOverrides()
    {
        $psOverridePath = _PS_OVERRIDE_DIR_ . 'modules/';
        $files = [
            'productcomments/controllers/front/PostComment.php',
            'iqitreviews/controllers/front/actions.php'
        ];
        foreach ($files as $file) {
            if (file_exists($psOverridePath . $file)) {
                unlink($psOverridePath . $file);
            }
        }
    }

    private function recursiveCopy($src, $dst)
    {
        if (!is_dir($dst)) {
            mkdir($dst, 0755, true);
        }
        $files = scandir($src);
        foreach ($files as $file) {
            if ($file !== '.' && $file !== '..') {
                $srcFile = $src . '/' . $file;
                $dstFile = $dst . '/' . $file;
                if (is_dir($srcFile)) {
                    $this->recursiveCopy($srcFile, $dstFile);
                } else {
                    copy($srcFile, $dstFile);
                    chmod($dstFile, 0755);
                }
            }
        }
    }

    private function performInstallQueries(array $queries)
    {
        foreach ($queries as $query) {
            Db::getInstance()->execute($query);
        }
    }

    public function getContent()
    {
        $this->context->smarty->assign('module_dir', $this->_path);
        $output = $this->context->smarty->fetch($this->local_path . 'views/templates/admin/configure.tpl');
        if ((Tools::isSubmit('submit' . $this->name)) == true) {
            foreach ($this->configNames() as $config) {
                Configuration::updateValue($config, (int) Tools::getValue($config));
            }
            foreach ($this->_att as $name => $action) {
                foreach (array_keys($action) as $actionName) {
                    $config = strtoupper($this->name . '_' . $name . '_' . $actionName);
                    $this->_att[$name][$actionName] = (int) Configuration::get($config);
                }
                if ($this->_att[$name]['warn'] >= $this->_att[$name]['ban']) {
                    $this->_errors[] = $this->l('Invalid Configuration values. BAN cannot be less than WARN') . ' ' . ucfirst($name);
                } 
            }
            if ($this->_errors) {
                foreach ($this->_errors as $e) {
                    $output .= $this->displayError($e);
                }
            } else {
                $output .= $this->displayConfirmation($this->l('Settings updated'));
            }
        }
        return $output . $this->displayForm();
    }

    public function displayForm()
    {
        $inputs = [
            [
                'type' => 'switch',
                'label' => $this->l('Hash in log stored passwords'),
                'name' => 'SIMPLESECURITY_PASSWORDS_HASH',
                'is_bool' => true,
                'desc' => $this->l('Hash in log stored passwords (enabled) or use clear text (disabled).'),
                'values' => [
                    [
                        'id' => 'hash_on',
                        'value' => true,
                        'label' => $this->l('Enabled')
                    ],
                    [
                        'id' => 'hash_off',
                        'value' => false,
                        'label' => $this->l('Disabled')
                    ]
                ]
            ]
        ];
        foreach ($this->_att as $name => $action) {
            foreach (array_keys($action) as $actionName) {
                $config = strtoupper($this->name . '_' . $name . '_' . $actionName);
                $inputs[] = [
                    'type' => 'text',
                    'label' => $this->l(ucfirst($name) . ' ' . ucfirst($actionName)),
                    'name' => $config,
                    'size' => 20,
                    'required' => true
                ];
            }
        }
        $form = [
            'form' => [
                'legend' => [
                    'title' => $this->l('Settings'),
                ],
                'input' => $inputs,
                'submit' => [
                    'title' => $this->l('Save'),
                    'class' => 'btn btn-default pull-right',
                ],
            ],
        ];
        $helper = new HelperForm();
        $helper->module = $this;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex . '&configure=' . $this->name;
        $helper->default_form_language = (int) Configuration::get('PS_LANG_DEFAULT');
        $helper->title = $this->displayName;
        $helper->submit_action = 'submit' . $this->name;
        foreach ($this->configNames() as $v) {
            $helper->fields_value[$v] = Configuration::get($v);
        }
        return $helper->generateForm([$form]);
    }

    private function configNames()
    {
        $names = [];
        $names[] = 'SIMPLESECURITY_PASSWORDS_HASH';
        foreach ($this->_att as $name => $action) {
            foreach (array_keys($action) as $actionName) {
                $names[] = strtoupper($this->name . '_' . $name . '_' . $actionName);
            }
        }
        return $names;
    }

    public function hookActionNewsletterRegistrationBefore($params)
    {
        $this->_action = 'newsletter';
        if (!$this->_executed) {
            $this->checkNewsletterReg();
        }
        $this->_executed = true;
        if ($this->_errors) {
            $params['hookError'] = implode(' ', $this->_errors);
        }
    }

    public function hookActionAuthenticationBefore($params)
    {
        $this->_redirect = $this->_action = 'authentication';
        $this->checkAuthAndReg();
    }

    public function hookActionSubmitAccountBefore($params)
    {
        $this->_redirect = $this->_action = 'registration';
        $this->checkAuthAndReg();
        if (!$this->_errors) {
            return true;
        }
    }

    private function checkConfig()
    {
        $configWarn = (int) Configuration::get(strtoupper($this->name . '_' . $this->_action . '_WARN'));
        $configBan = (int) Configuration::get(strtoupper($this->name . '_' . $this->_action . '_BAN'));
        if ($configWarn < $configBan) {
            return true;
        }
    }

    public function checkNewsletterReg()
    {
        $this->_email = isset($_POST['email']) ? pSQL($_POST['email']) : null;
        if (!$this->_email || !$this->checkConfig()) {
            return;
        }
        $this->_remote_addr = $_SERVER['REMOTE_ADDR'] ?? null;
        if (!$this->checkIsBlocked()) {
            $this->checkIsBot(true);
            $this->_attempt = $this->getAttemptsCount();
            if (($this->_attempt >= $this->_att[$this->_action]['warn']) &&
                ($this->_attempt < $this->_att[$this->_action]['ban'])) {
                if ($this->_block_ip !== 2 && $this->_block_email !== 2) {
                    $this->_errors[] = $this->l('Temporarily prohibited. Please try again in a few minutes.');
                }
                $this->_detected[] = 'warn';
            }
            if ($this->_attempt >= $this->_att[$this->_action]['ban']) {
                if ($this->_block_ip !== 2 && $this->_block_email !== 2) {
                    $this->_errors[] = $this->l('Prohibited. Please contact site administrator.');
                    $this->_block_ip = 1;
                }
                $this->_detected[] = 'ban';
            }
            $this->storeData();
        }
        return $this->_errors;
    }

    private function checkAuthAndReg()
    {
        $this->_email = isset($_POST['email']) ? pSQL($_POST['email']) : null;
        $this->_password = isset($_POST['password']) ? pSQL($_POST['password'], true) : null;
        if (!$this->_password || !$this->_email || !$this->checkConfig()) {
            return;
        }
        $this->_remote_addr = $_SERVER['REMOTE_ADDR'] ?? null;
        if (!$this->checkIsBlocked()) {
            $this->checkIsBot();
            $this->_attempt = $this->getAttemptsCount();
            if (($this->_attempt >= $this->_att[$this->_action]['warn']) &&
                ($this->_attempt < $this->_att[$this->_action]['ban'])) {
                if ($this->_block_ip !== 2 && $this->_block_email !== 2) {
                    $this->_errors[] = $this->l('Temporarily prohibited. Please try again in a few minutes.');
                }
                $this->_detected[] = 'warn';
            }
            if ($this->_attempt >= $this->_att[$this->_action]['ban']) {
                if ($this->_block_ip !== 2 && $this->_block_email !== 2) {
                    $this->_errors[] = $this->l('Prohibited. Please contact site administrator.');
                    $this->_block_ip = 1;
                }
                $this->_detected[] = 'ban';
            }
            $this->storeData();
        }
        if (!$this->_errors) {
            return;
        } else {
            $this->context->controller->errors = array_merge($this->context->controller->errors, $this->_errors);
            if ($this->_redirect) {
                die($this->context->controller->redirectWithNotifications('index.php?controller=' . $this->_redirect));
            }
        }
    }

    public function checkContact()
    {
        $this->_email = isset($_POST['from']) ? pSQL($_POST['from']) : null;
        $this->_message = isset($_POST['message']) ? pSQL($_POST['message'], true) : null;
        if (!$this->checkConfig()) {
            return [];
        }
        $this->_remote_addr = $_SERVER['REMOTE_ADDR'] ?? null;
        if (!$this->checkIsBlocked()) {
            $this->checkIsBot();
            $this->_attempt = $this->getAttemptsCount();
            if (($this->_attempt >= $this->_att[$this->_action]['warn']) &&
                ($this->_attempt < $this->_att[$this->_action]['ban'])) {
                if ($this->_block_ip !== 2 && $this->_block_email !== 2) {
                    $this->_errors[] = $this->l('Temporarily prohibited. Please try again in a few minutes.');
                }
                $this->_detected[] = 'warn';
            }
            if ($this->_attempt >= $this->_att[$this->_action]['ban']) {
                if ($this->_block_ip !== 2 && $this->_block_email !== 2) {
                    $this->_errors[] = $this->l('Prohibited. Please contact site administrator.');
                    $this->_block_ip = 1;
                }
                $this->_detected[] = 'ban';
            }
            if ($this->hasStopWord()) {
                if ($this->_block_ip !== 2 && $this->_block_email !== 2) {
                    $this->_errors[] = $this->l('Please not spamming here.');
                }
                $this->_detected[] = 'spam';
            }
            if (!preg_match("/\[$this->_remote_addr\]/", $this->_message)) {
                $_POST['message'] = '[' . $this->_remote_addr . ']' . "\n" . $this->_message;
            }
            $this->storeData();
        }
        return $this->_errors;
    }

    public function checkComment()
    {
        $this->_message = isset($_POST['comment_content']) ? pSQL($_POST['comment_content']) : null;
        $this->_message .= isset($_POST['comment_title']) ? ' ' . pSQL($_POST['comment_title']) : null;
        $this->_message .= isset($_POST['customer_name']) ? ' ' . pSQL($_POST['customer_name']) : null;
        if (!$this->_message || !$this->checkConfig()) {
            return;
        }
        $this->_remote_addr = $_SERVER['REMOTE_ADDR'] ?? null;
        if (!$this->checkIsBlocked()) {
            $this->checkIsBot(true);
            $this->_attempt = $this->getAttemptsCount();
            if (($this->_attempt >= $this->_att[$this->_action]['warn']) &&
                ($this->_attempt < $this->_att[$this->_action]['ban'])) {
                if ($this->_block_ip !== 2) {
                    $this->_errors[] = $this->l('Temporarily prohibited. Please try again in a few minutes.');
                }
                $this->_detected[] = 'warn';
            }
            if ($this->_attempt >= $this->_att[$this->_action]['ban']) {
                if ($this->_block_ip !== 2) {
                    $this->_errors[] = $this->l('Prohibited. Please contact site administrator.');
                    $this->_block_ip = 1;
                }
                $this->_detected[] = 'ban';
            }
            if ($this->hasStopWord()) {
                if ($this->_block_ip !== 2) {
                    $this->_errors[] = $this->l('Please not spamming here.');
                }
                $this->_detected[] = 'spam';
            }
            $this->storeData();
        }
        return $this->_errors;
    }

    public function checkIqitReview()
    {
        $this->_message = isset($_POST['iqitreviews_comment']) ? pSQL($_POST['iqitreviews_comment']) : null;
        $this->_message .= isset($_POST['iqitreviews_title']) ? ' ' . pSQL($_POST['iqitreviews_title']) : null;
        $this->_message .= isset($_POST['iqitreviews_customer_name']) ? ' ' . pSQL($_POST['iqitreviews_customer_name']) : null;
        if (!$this->_message || !$this->checkConfig()) {
            return;
        }
        $this->_remote_addr = $_SERVER['REMOTE_ADDR'] ?? null;
        if (!$this->checkIsBlocked()) {
            $this->checkIsBot(true);
            $this->_attempt = $this->getAttemptsCount();
            if (($this->_attempt >= $this->_att[$this->_action]['warn']) &&
                ($this->_attempt < $this->_att[$this->_action]['ban'])) {
                if ($this->_block_ip !== 2) {
                    $this->_errors[] = $this->l('Temporarily prohibited. Please try again in a few minutes.');
                }
                $this->_detected[] = 'warn';
            }
            if ($this->_attempt >= $this->_att[$this->_action]['ban']) {
                if ($this->_block_ip !== 2) {
                    $this->_errors[] = $this->l('Prohibited. Please contact site administrator.');
                    $this->_block_ip = 1;
                }
                $this->_detected[] = 'ban';
            }
            if ($this->hasStopWord()) {
                if ($this->_block_ip !== 2) {
                    $this->_errors[] = $this->l('Please not spamming here.');
                }
                $this->_detected[] = 'spam';
            }
            $this->storeData();
        }
        return $this->_errors;
    }

    private function checkIsBlocked()
    {
        if ($this->getIpBlocked()) {
            if ($this->_block_ip === 1) {
                $this->_errors[] = $this->l('IP address blocked. Please contact site administrator.');
                $return = true;
            }
        }
        if ($this->getEmailBlocked()) {
            if ($this->_block_email === 1) {
                $this->_errors[] = $this->l('Email address blocked. Please contact site administrator.');
                $return = true;
            }
        }
        return $return ?? false;
    }

    private function checkIsBot($xhr = false)
    {
        $this->_http_x_requested_with = $_SERVER['HTTP_X_REQUESTED_WITH'] ?? null;
        $this->_http_referer = $_SERVER['HTTP_REFERER'] ?? null;
        $this->_http_user_agent = $_SERVER['HTTP_USER_AGENT'] ?? null;
        $this->_http_accept = $_SERVER['HTTP_ACCEPT'] ?? null;
        $this->_http_accept_language = $_SERVER['HTTP_ACCEPT_LANGUAGE'] ?? null;
        $this->_http_cookie = $_SERVER['HTTP_COOKIE'] ?? null;
        $this->_http_host = $_SERVER['HTTP_HOST'] ?? null;
        $this->_php_self = $_SERVER['PHP_SELF'] ?? null;
        $this->_script_name = $_SERVER['SCRIPT_NAME'] ?? null;
        $checks = [
            ($xhr && ($this->_http_x_requested_with == 'XMLHttpRequest')) ? (strstr($this->_http_accept, '*/*')) : strstr($this->_http_accept, 'text/html'),
            preg_match('/[a-z]{2,3}/i', $this->_http_accept_language),
            strstr($this->_http_cookie, 'Shop-'),
            strstr($this->_http_cookie, 'PHPSESSID'),
            strstr($this->_http_referer, $this->_http_host),
            strstr($this->_http_user_agent, 'Mozilla/5.0'),
            ($this->_script_name == '/index.php') ? true : false,
            ($this->_php_self == '/index.php') ? true : false
        ];
        foreach ($checks as $checkPassed) {
            if (!$checkPassed) {
                $this->_detected[] = 'bot';
                if ($this->_block_ip !== 2) {
                    $this->_errors[] = $this->l('Bot or invalid traffic detected. Connection prohibited.');
                    $this->_block_ip = 1;
                }
                return true;
            }
        }
    }

    private function storeData()
    {
        if ($this->_block_ip === 2 || $this->_block_email === 2) {
            return;
        }
        $password = ($this->pwd_use_hash && !empty($this->_password)) ? Tools::hash($this->_password) : $this->_password;
        Db::getInstance()->execute(
            'INSERT IGNORE INTO `' . _DB_PREFIX_ . $this->name . '_action` (
                `action`,
                `attempt`,
                `detected`,
                `email`,
                `block_email`,
                `password`,
                `ip_address`,
                `block_ip`,
                `referer`,
                `user_agent`,
                `accept`,
                `accept_lang`, 
                `cookie`
            ) VALUES (
                "' . $this->_action . '",
                "' . $this->_attempt . '",
                "' . implode(',', $this->_detected) . '",
                "' . $this->_email . '",
                "' . $this->_block_email . '",
                "' . $password . '",
                "' . $this->_remote_addr . '",
                "' . $this->_block_ip . '",
                "' . $this->_http_referer . '",
                "' . $this->_http_user_agent . '",
                "' . $this->_http_accept . '",
                "' . $this->_http_accept_language . '",
                "' . $this->_http_cookie . '"
            )'
        );
    }

    private function getIpBlocked()
    {
        return $this->_block_ip = (int) Db::getInstance()->getValue(
            "SELECT `block_ip` FROM `" . _DB_PREFIX_ . $this->name . "_action`
            WHERE `action` = '" . $this->_action . "' AND `ip_address` LIKE '" . $this->_remote_addr . "'
            ORDER BY `date` DESC"
        );
    }

    private function getEmailBlocked()
    {
        if ($this->_email) {
            return $this->_block_email = (int) Db::getInstance()->getValue(
                "SELECT `block_email` FROM `" . _DB_PREFIX_ . $this->name . "_action`
                WHERE `action` = '" . $this->_action . "' AND `email` LIKE '" . $this->_email . "'
                ORDER BY `date` DESC"
            );
        }
    }

    private function getAttemptsCount()
    {
        return $this->_attempt += (int) Db::getInstance()->getValue(
            "SELECT COUNT(*) FROM `" . _DB_PREFIX_ . $this->name . "_action`
            WHERE `action` = '" . $this->_action. "' AND `ip_address` LIKE '" . $this->_remote_addr . "'
            AND `date` > (now() - interval " . $this->_checkAttemptsIntervalMinutes . " minute)"
        );
    }

    private function hasStopWord()
    {
        $msg = strtolower($this->_message);
        if ($result = Db::getInstance()->executeS("SELECT `words` FROM `" . _DB_PREFIX_ . $this->name . "_trigger`")) {
            foreach ($result as $w) {
                if (strpos($msg, $w['words']) !== FALSE) {
                    return true;
                }
            }
        }
    }
}
